/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.player;

import java.io.*;
import java.util.*;
import java.util.regex.*;
import java.net.*;
import melodia.util.*;

/**
 *
 * @author Wyverald
 */
public class Lyrics {
  private ArrayList<Line> lines;
  private HashMap<String, String> idMap;
  private double offset = 0;
  
  private final static Pattern idPattern =
      Pattern.compile("\\[(ar|au|al|by|ti|re|ve):([^\\]]*)\\]");
  private final static Pattern offsetPattern =
      Pattern.compile("\\[offset:(\\d+)\\]");
  private final static Pattern lyricLinePattern =
      Pattern.compile("((?:\\[\\d+:\\d\\d?(?:\\.\\d+)?\\])+)(.*)");
  private final static Pattern lyricTimestampPattern = 
      Pattern.compile("\\[(\\d+):(\\d\\d?(?:\\.\\d+)?)\\]");
  
  public Lyrics() {
    lines = new ArrayList<Line>();
  }
  
  public Lyrics(File lrcFile) {
    TextFileReader reader = null;
    try {
      lines = new ArrayList<Line>();
      idMap = new HashMap<String, String>();
      reader = new TextFileReader(lrcFile);
      String line = null;
      while ((line = reader.readLine()) != null) {
        parseLine(line);
      }
      Collections.sort(lines);
      
      if (lines.isEmpty()) lines.add(new Line("Lyrics not available", 0));
      reader.close();
    } catch (FileNotFoundException ex) {
      lines.add(new Line("Lyrics not available", 0));
      System.err.println("Lyrics file not found: " + lrcFile);
    } catch (IOException ex) {
      lines.add(new Line("Error loading lyrics", 0));
    }
    
    // Find lengths of all lines of lyrics.
    for (int i = 0; i < lines.size(); i++) {
      lines.get(i).length = 
          (i + 1 == lines.size() ? 1e100 : lines.get(i + 1).time)
          - lines.get(i).time;
    }
  }
  
  public static File findLyricsFileFromSongFile(File musicFile) {
    int lastDot = musicFile.getName().lastIndexOf(".");
    String lrcFilename = null;
    if (lastDot == -1) {
      lrcFilename = musicFile.getName() + ".lrc";
    } else {
      lrcFilename = musicFile.getName().substring(0, lastDot) + ".lrc";
    }
    return new File(musicFile.getParent(), lrcFilename);
  }
  
  public static Lyrics findLyricsFromSongFile(File musicFile) {
    File lyricsFile = findLyricsFileFromSongFile(musicFile);
    if (lyricsFile.exists()) return new Lyrics(lyricsFile);
    
    // In lieu of a local lyrics file, we search online using Baidu's mp3
    // search engine.
    SongMetadata data = new SongMetadata(musicFile);
    try {
      BufferedReader reader = new BufferedReader(new InputStreamReader(
          new URL(String.format(
          "http://box.zhangmen.baidu.com/x?op=12&count=1&title=%s$$%s$$$$",
          data.getTitle(), data.getArtist())).openStream()));
      String result = reader.readLine();
      reader.close();
      
      int lstart = result.indexOf("<lrcid>"), lend = result.indexOf("</lrcid>");
      if (lstart < 0) return new Lyrics(lyricsFile);
      int lrcid = Integer.parseInt(result.substring(lstart + 7, lend));
      if (lrcid == 0) return new Lyrics(lyricsFile);
      
      InputStream is = new URL(String.format(
          "http://box.zhangmen.baidu.com/bdlrc/%d/%d.lrc", lrcid / 100, lrcid))
          .openStream();
      OutputStream os = new FileOutputStream(lyricsFile);
      
      byte[] buf = new byte[8192];
      int bytesRead;
      while ((bytesRead = is.read(buf)) > 0) {
        os.write(buf, 0, bytesRead);
      }
      os.close();
      is.close();
      
      return new Lyrics(lyricsFile);
    } catch (IOException ex) {
      return new Lyrics(lyricsFile);
    }
  }

  /**
   * Parse a physical line from the lyrics file. Depending on the format
   * of the line, it could signify an ID detail, a global offset adjustment,
   * or an actual lyric line. This method accordingly adds the lyric line
   * into a line pool or remembers the ID detail/offset adjustment.
   * @param line The physical line to be parsed.
   */
  protected final void parseLine(String line) {
    line = line.trim();
    
    Matcher idMatcher = idPattern.matcher(line);
    if (idMatcher.matches()) {
      idMap.put(idMatcher.group(1), idMatcher.group(2));
      return;
    }
    
    Matcher offsetMatcher = offsetPattern.matcher(line);
    if (offsetMatcher.matches()) {
      offset = (double) Integer.parseInt(offsetMatcher.group(1)) / 1000;
      return;
    }
    
    Matcher lyricLineMatcher = lyricLinePattern.matcher(line);
    if (lyricLineMatcher.matches()) {
      String timestamps = lyricLineMatcher.group(1),
             lyrics = lyricLineMatcher.group(2);
      Matcher lyricTSMatcher = lyricTimestampPattern.matcher(timestamps);
      while (lyricTSMatcher.find()) {
        lines.add(new Line(lyrics, Double.parseDouble(lyricTSMatcher.group(2)) +
            60 * Integer.parseInt(lyricTSMatcher.group(1)) + offset));
      }
    }
  }
  
  public ArrayList<Line> getLines() { return lines; }
  
  /** Get the author of the song.
   * @return The author of the song.
   */
  public String getAuthor() { return idMap.get("au"); }
  /** Get the album name of the song.
   * @return The album name of the song.
   */
  public String getAlbum() { return idMap.get("al"); }
  /** Get the artist performing the song.
   * @return The artist performing the song.
   */
  public String getArtist() { return idMap.get("ar"); }
  /** Get the title of the song.
   * @return The title of the song.
   */
  public String getTitle() { return idMap.get("ti"); }
  /** Get the maker of this lyrics file.
   * @return The maker of this lyrics file.
   */
  public String getMadeBy() { return idMap.get("by"); }
  /** Get the name of the software this lyrics file is made with.
   * @return The name of the software this lyrics file is made with.
   */
  public String getSoftware() { return idMap.get("re"); }
  /** Get the version of the software this lyrics file is made with.
   * @return The version of the software this lyrics file is made with.
   */
  public String getVersion() { return idMap.get("ve"); }
  
  public int getLineIndexFromTime(double time) {
    int index = Collections.binarySearch(lines, new Line("", time));
    if (index >= 0) return index;
    else return -index - 2;
  }

  /**
   * Denotes a line of lyrics. It specifies the actual lyrics of this line,
   * along with the timestamp (in seconds) when it should be sung.
   * 
   * Note: this class has a natural ordering that is inconsistent with equals.
   */
  public class Line implements Comparable<Line> {
    /** The actual lyrics of this line. */
    public String lyrics;
    /** The timestamp, in seconds, of this line. */
    public double time;
    /** The length of this line. */
    public double length;
    /** Constructs a Line from its lyrics and timestamp.
     * @param actualLyrics The actual lyrics of this line.
     * @param timeStamp The timestamp, in seconds, of this line.
     */
    public Line(String actualLyrics, double timeStamp) {
      lyrics = actualLyrics;
      time = timeStamp;
    }

    /** Compare to another Line in time.
     * @param t The line to compare to.
     * @return -1 if this line is ahead of the other line; 1 if this line is
     * after the other line; 0 otherwise.
     */
    @Override
    public int compareTo(Line t) {
      if (time < t.time) return -1;
      else if (time > t.time) return 1;
      else return 0;    // We all know that this is highly unlikely...
    }
  }
}
