/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.player;

import javax.media.*;
import java.io.*;

/**
 *
 * @author Wyverald
 */
public class PlayerManager {
  private Player player;
  private double duration = 0;
  private PlayerListener listener;
  private Song currentSong;

  public static String formatTime(double time) {
    return String.format("%02d:%02d", (int) (time / 60), (int) (time % 60));
  }
  
  public PlayerManager(PlayerListener pl) {
    listener = pl;
    currentSong = null;
  }
  
  public boolean isPlaying() {
    return player != null && player.getState() == Player.Started;
  }
  
  public void dispose() {
    if (player != null) {
      player.stop();
      player.deallocate();
      listener.oldPlayerDisposed(currentSong);
      currentSong = null;
      player = null;
    }
  }
  
  public void setVolume(double gain) {
    if (player != null) player.getGainControl().setLevel((float) gain);
  }
  
  public double getDuration() {
    return duration;
  }
  
  public double getCurrentTime() {
    return player == null ? 0 : player.getMediaTime().getSeconds();
  }
  
  public Song getCurrentSong() {
    return currentSong;
  }
  
  public void setProgress(double perc) {
    if (player != null) player.setMediaTime(new Time(perc * duration));
  }
  
  public double getProgress() {
    return duration == 0 ? 0 : getCurrentTime() / duration;
  }
  
  public void start() {
    if (player != null) player.start();
  }
  
  public void stop() {
    if (player != null) player.stop();
  }
  
  public void switchSong(Song song) {
    try {
      dispose();
      listener.beforeNewPlayerCreated(song);
      
      currentSong = song;
      if (song == null) return;
      player = Manager.createPlayer(song.getFile().toURI().toURL());
      player.addControllerListener(new ControllerListener() {

        @Override
        public void controllerUpdate(final ControllerEvent ce) {
          javax.swing.SwingUtilities.invokeLater(new Runnable() {

            @Override
            public void run() {
              PlayerManager.this.controllerUpdate(ce);
            }
          });
        }
      });
      duration = 0;
      
      player.start();
    } catch (IOException ex) {
      System.out.println("FILE NOT FOUND!!!");
    } catch (NoPlayerException ex) {
      System.out.println("WTF NOPLAYER EXCEPTION???");
    }
  }

  private void controllerUpdate(ControllerEvent ce) {
    if (ce instanceof EndOfMediaEvent) {
      listener.playerStopped();
      listener.endOfMediaReached(currentSong);
    } else if (ce instanceof StopEvent) {
      listener.playerStopped();
    } else if (ce instanceof StartEvent) {
      listener.playerStarted();
    } else if (ce instanceof DurationUpdateEvent) {
      duration = ((DurationUpdateEvent) ce).getDuration().getSeconds();
      listener.durationUpdated();
    }
  }
}
