/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.player;

import java.io.File;
import java.util.*;
import org.json.*;
import melodia.util.Config;

/**
 *
 * @author Wyverald
 */
public class PlaylistManager {
  private ArrayList<Song> songs, randomizedSongs;
  PlaylistListener listener;
  private ArrayList<Playlist> playlists;
  
  @SuppressWarnings("OverridableMethodCallInConstructor")
  public PlaylistManager() {
    songs = new ArrayList<Song>();
    randomizedSongs = new ArrayList<Song>();
    playlists = new ArrayList<Playlist>();
  }
  
  public PlaylistManager(PlaylistListener listener) {
    this();
    this.listener = listener;
  }
  
  public void clearPlaylist() {
    songs.clear();
    randomizedSongs.clear();
    if (listener != null) listener.playlistCleared();
  }
  
  public void savePlaylist() {
    ArrayList<File> files = new ArrayList<File>();
    for (Iterator<Song> it = songs.iterator(); it.hasNext();) {
      Song song = it.next();
      if (song.isTemporary()) continue;
      files.add(song.getFile());
    }
    
    Playlist pl = new Playlist("[New Playlist]", files);
    addSavedPlaylist(pl);
    renameSavedPlaylist(pl);
  }
  
  public void loadSavedPlaylists() {
    JSONObject co = Config.getInstance().getConfigObject();
    try {
      // Load all playlists from the config file.
      JSONArray pla = co.getJSONArray("playlists");
      for (int i = 0; i < pla.length(); i++) {
        JSONObject plobj = pla.getJSONObject(i);
        addSavedPlaylist(new Playlist(plobj.getString("name"),
            plobj.getJSONArray("files")));
      }
    } catch (JSONException ex) {
      System.err.println("Error reading playlists");
    }
    
    try {
      // Load last played songs.
      JSONArray lap = co.getJSONArray("last-played-songs");
      for (int i = 0; i < lap.length(); i++) {
        addSong(new Song(new File(lap.getString(i))));
      }
    } catch (JSONException ex) {
      System.err.println("Error reading last played songs");
    }
  }
  
  public void addSavedPlaylist(Playlist playlist) {
    playlists.add(playlist);
    if (listener != null) listener.playlistAdded(playlist);
  }
  
  public void appendSavedPlaylist(Playlist playlist) {
    for (File file : playlist.getFiles()) {
      addSong(new Song(file));
    }
  }
  
  public void playSavedPlaylist(Playlist playlist) {
    clearPlaylist();
    appendSavedPlaylist(playlist);
  }
  
  public void removeSavedPlaylist(Playlist playlist) {
    playlists.remove(playlist);
    if (listener != null) listener.playlistRemoved(playlist);
  }
  
  public void renameSavedPlaylist(Playlist playlist) {
    playlist.getButton().performRename();
  }
  
  public Song getSong(int i) {
    return songs.get(i);
  }
  
  public int getNumberOfSongs() {
    return songs.size();
  }

  public void addSong(Song song) {
    songs.add(song);
    randomizedSongs.add(song);
    Collections.shuffle(randomizedSongs);
    if (listener != null) listener.songAdded(song);
  }
  
  public void removeSong(Song song) {
    songs.remove(song);
    randomizedSongs.remove(song);
    if (listener != null) listener.songRemoved(song);
  }
  
  public Song getNextSong(Song song, int loopMode, boolean rand) {
    ArrayList<Song> songList = rand ? randomizedSongs : songs;
    if (song == null || songList.isEmpty()) return null;
    if (loopMode == 2) return song;
    
    for (Iterator<Song> it = songList.iterator(); it.hasNext();) {
      if (song == it.next()) {
        if (it.hasNext()) return it.next();
        else return loopMode == 1 ? songList.get(0) : null;
      }
    }
    
    // How did you get here...?
    // ... Do I know you...?
    return null;
  }

  public Song getPreviousSong(Song currentSong, int loopMode, boolean rand) {
    ArrayList<Song> songList = rand ? randomizedSongs : songs;
    if (currentSong == null || songList.isEmpty()) return null;
    if (loopMode == 2) return currentSong;
    
    Song lastSong = loopMode == 1 ? songList.get(songList.size() - 1) : null;
    for (Iterator<Song> it = songList.iterator(); it.hasNext();) {
      Song song = it.next();
      if (currentSong == song) {
        return lastSong;
      }
      lastSong = song;
    }
    
    // Where am I...?
    // ... Who... am I...?
    return null;
  }
  
  public void storeConfig() {
    try {
      JSONArray ja = new JSONArray();
      for (Iterator<Playlist> it = playlists.iterator(); it.hasNext();) {
        Playlist playlist = it.next();
      
        JSONObject jo = new JSONObject();
        jo.put("name", playlist.getName());
        jo.put("files", playlist.getJSONArray());
        ja.put(jo);
      }
      Config.getInstance().getConfigObject().put("playlists", ja);
    } catch (JSONException ex) {
      System.err.println("Error storing playlists");
    }
    
    try {
      JSONArray ja = new JSONArray();
      for (Iterator<Song> it = songs.iterator(); it.hasNext();) {
        Song song = it.next();
        if (song.isTemporary()) continue;
        ja.put(song.getFile().getAbsolutePath());
      }
      Config.getInstance().getConfigObject().put("last-played-songs", ja);
    } catch (JSONException ex) {
      System.err.println("Error storing last played songs");
    }
  }
  
}
