/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.player;

import java.awt.Image;
import java.io.*;
import javax.imageio.ImageIO;
import melodia.util.ResourcePreloader;

/**
 *
 * @author Wyverald
 */
public class SongMetadata {
  private String artist, album, title, year, track, genre;
  private Image coverArt;
  
  public SongMetadata(File file) {
    try {
      RandomAccessFile raf = new RandomAccessFile(file, "r");
      
      readID3v2(raf);
      //readID3v1(raf);
    } catch (FileNotFoundException ex) {
      System.err.println("Error: File not found");
    } catch (IOException ex) {
      System.err.println("Error reading tags from file");
    }

    if (artist == null || artist.isEmpty()) artist = "Unknown Artist";
    if (album == null || album.isEmpty()) album = "Unknown Album";
    if (title == null || title.isEmpty()) title = "Unknown Title";
    if (year == null || year.isEmpty()) year = "Unknown";
    else if (year.length() >= 4) year = year.substring(0, 4);
    if (track == null || track.isEmpty()) track = "Unknown";
    if (genre == null || genre.isEmpty()) genre = "Unknown";
    if (coverArt == null) coverArt = ResourcePreloader.getDefaultCoverArt();
  }
  
  private void readID3v2(RandomAccessFile raf) throws IOException {
    raf.seek(0);
    if (raf.readInt() != 0x49443303) return;
    
    raf.readChar();   // skip two bytes
    long size = 0;
    size |= raf.readByte() << 21;
    size |= raf.readByte() << 14;
    size |= raf.readByte() << 7;
    size |= raf.readByte();
    
    while (raf.getFilePointer() <= size) readID3v2Frame(raf);
  }
  
  private void readID3v2Frame(RandomAccessFile raf) throws IOException {
    byte[] frameIDBuf = new byte[4];
    raf.readFully(frameIDBuf);
    String frameID = new String(frameIDBuf);
    
    int size = raf.readInt();
    raf.readChar();     // Skip two bytes.
    
    if (frameID.equals("TIT2")) {
      title = readEncodedString(raf, size);
    } else if (frameID.equals("TALB")) {
      album = readEncodedString(raf, size);
    } else if (frameID.equals("TPE1")) {
      artist = readEncodedString(raf, size);
    } else if (frameID.equals("TRCK")) {
      track = readEncodedString(raf, size);
    } else if (frameID.equals("TYER")) {
      year = readEncodedString(raf, size);
    } else if (frameID.equals("APIC")) {
      coverArt = readCoverArt(raf, size);
    } else if (frameID.equals("TCON")) {
      genre = decodeGenre(readEncodedString(raf, size));
    } else {
      // Ignore this tag.
      raf.seek(raf.getFilePointer() + size);
    }
  }
  
  private String readEncodedString(RandomAccessFile raf, int size) throws IOException {
    byte[] buf = new byte[size - 1];
    boolean utf16 = raf.readByte() == 1;
    raf.readFully(buf);
    return new String(buf, utf16 ? "UTF-16" : "GBK");
  }
  
  private Image readCoverArt(RandomAccessFile raf, int size) throws IOException {
    boolean utf16 = raf.readByte() == 1;
    size--;
    
    String mime;
    /* Read mime type */ {
      StringBuilder sb = new StringBuilder();
      byte b;
      while (true) {
        b = raf.readByte();
        size--;
        if (b == 0) break;
        sb.appendCodePoint(b);
      }
      mime = sb.toString();
    }
    
    int imageType = raf.readByte();
    size--;
    
    // Read image description.
    String desc;
    if (utf16) {
      byte[] buf = new byte[size];
      int pointer = 0;
      byte b1, b2;
      while (true) {
        b1 = raf.readByte();
        b2 = raf.readByte();
        size -= 2;
        if (b1 == 0 && b2 == 0) break;
        buf[pointer++] = b1;
        buf[pointer++] = b2;
      }
      desc = new String(buf, "UTF-16");
    } else {
      byte[] buf = new byte[size];
      int pointer = 0;
      byte b;
      while (true) {
        b = raf.readByte();
        size--;
        if (b == 0) break;
        buf[pointer++] = b;
      }
      desc = new String(buf, "GBK");
    }
    
    // Read actual binary data.
    byte[] buf = new byte[size];
    raf.readFully(buf);
    return ImageIO.read(new ByteArrayInputStream(buf));
  }
  
  public static final String[] GENRES = {
    "Blues", "Classic Rock", "Country", "Dance", "Disco",
    "Funk", "Grunge", "Hip-Hop", "Jazz", "Metal",
    "New Age", "Oldies", "Other", "Pop", "R&B",
    "Rap", "Reggae", "Rock", "Techno", "Industrial",
    "Alternative", "Ska", "Death Metal", "Pranks", "Soundtrack",
    "Euro-Techno", "Ambient", "Trip-Hop", "Vocal", "Jazz+Funk",
    "Fusion", "Trance", "Classical", "Instrumental", "Acid",
    "House", "Game", "Sound Clip", "Gospel", "Noise",
    "AlternRock", "Bass", "Soul", "Punk", "Space",
    "Meditative", "Instrumental Pop", "Instrumental Rock", "Ethnic", "Gothic",
    "Darkwave", "Techno-Industrial", "Electronic", "Pop-Folk", "Eurodance",
    "Dream", "Southern Rock", "Comedy", "Cult", "Gangsta",
    "Top 40", "Christian Rap", "Pop/Funk", "Jungle", "Native American",
    "Cabaret", "New Wave", "Psychadelic", "Rave", "Showtunes",
    "Trailer", "Lo-Fi", "Tribal", "Acid Punk", "Acid Jazz",
    "Polka", "Retro", "Musical", "Rock & Roll", "Hard Rock",
    "Folk", "Folk-Rock", "National Folk", "Swing", "Fast Fusion",
    "Bebob", "Latin", "Revival", "Celtic", "Bluegrass", "Avantgarde",
    "Gothic Rock", "Progressive Rock", "Psychedelic Rock", "Symphonic Rock",
    "Slow Rock", "Big Band", "Chorus", "Easy Listening", "Acoustic",
    "Humour", "Speech", "Chanson", "Opera", "Chamber Music",
    "Sonata", "Symphony", "Booty Bass", "Primus", "Porn Groove",
    "Satire", "Slow Jam", "Club", "Tango", "Samba", "Folklore",
    "Ballad", "Power Ballad", "Rhythmic Soul", "Freestyle", "Duet",
    "Punk Rock", "Drum Solo", "A capella", "Euro-House", "Dance Hall"
  };
  
  private String decodeGenre(String raw) {
    if (raw.startsWith("(")) {
      return GENRES[Integer.parseInt(raw.substring(1, raw.charAt(3) == ')' ? 3 : 4))];
    } else {
      return raw;
    }
  }
  
  public String getArtist() { return artist; }
  
  public String getAlbum() { return album; }
  
  public String getTitle() { return title; }
  
  public String getYear() { return year; }
  
  public String getTrackNumber() { return track; }
  
  public String getGenre() { return genre; }
  
  public Image getCoverArt() { return coverArt; }
}
