/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.player.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.io.File;
import java.util.*;
import javax.swing.*;
import melodia.player.Playlist;
import melodia.player.SongMetadata;
import melodia.util.ResourcePreloader;

/**
 *
 * @author Wyverald
 */
public class PlaylistButton extends ListButton {
  private Playlist playlist;
  private JLabel lblName, lblNumSongs, lblCoverArts;
  private JTextField txtName;
  private JPanel pnlInfo;
  private SubButton btnRemove, btnRename, btnAdd, btnPlay;
  private CoverArtPainter worker;
  
  @SuppressWarnings("OverridableMethodCallInConstructor")
  public PlaylistButton(Playlist playlist){
    this.playlist = playlist;
    setAlignmentY(0.5f);
    setActualSize(new Dimension(270, 135));
    setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
    
    lblName = new JLabel();
    lblName.setAlignmentX(0);
    txtName = new JTextField(12);
    txtName.setAlignmentX(0);
    txtName.setVisible(false);
    txtName.setPreferredSize(new Dimension(120, 28));
    txtName.setSize(new Dimension(120, 28));
    txtName.setMinimumSize(new Dimension(120, 28));
    txtName.setMaximumSize(new Dimension(120, 28));
    lblNumSongs = new JLabel();
    lblNumSongs.setAlignmentX(0);
    lblCoverArts = new JLabel();
    lblCoverArts.setAlignmentX(0);
    lblCoverArts.setAlignmentY(0);
    lblCoverArts.setPreferredSize(new Dimension(120, 120));
    lblCoverArts.setSize(new Dimension(120, 120));
    lblCoverArts.setMinimumSize(new Dimension(120, 120));
    lblCoverArts.setMaximumSize(new Dimension(120, 120));
    JPanel pnlButtons = new JPanel();
    pnlButtons.setLayout(new BoxLayout(pnlButtons, BoxLayout.LINE_AXIS));
    pnlButtons.setAlignmentX(0);
    pnlButtons.setOpaque(false);
    btnRemove = new SubButton(this, "Remove this playlist", new ImageIcon(getClass()
        .getResource("/melodia/player/gui/resources/cross-script.png")));
    btnRename = new SubButton(this, "Rename this playlist", new ImageIcon(getClass()
        .getResource("/melodia/player/gui/resources/document-rename.png")));
    btnAdd = new SubButton(this, "Add to current playlist", new ImageIcon(getClass()
        .getResource("/melodia/player/gui/resources/plus.png")));
    btnPlay = new SubButton(this, "Play this playlist", new ImageIcon(getClass()
        .getResource("/melodia/player/gui/resources/control.png")));
    
    pnlInfo = new JPanel();
    pnlInfo.setLayout(new BoxLayout(pnlInfo, BoxLayout.PAGE_AXIS));
    pnlInfo.setOpaque(false);
    pnlInfo.setMaximumSize(new Dimension(120, 135));
    pnlInfo.setPreferredSize(new Dimension(120, 135));
    pnlInfo.setAlignmentY(0);
    pnlInfo.add(lblName);
    pnlInfo.add(txtName);
    pnlInfo.add(lblNumSongs);
    pnlInfo.add(Box.createVerticalGlue());
    pnlButtons.add(Box.createHorizontalGlue());
    pnlButtons.add(btnRemove);
    pnlButtons.add(btnRename);
    pnlButtons.add(btnAdd);
    pnlButtons.add(btnPlay);
    pnlInfo.add(pnlButtons);
    
    add(lblCoverArts);
    add(Box.createRigidArea(new Dimension(10, 0)));
    add(pnlInfo);
    updateView();
    setHighlight(false);
    
    txtName.addFocusListener(new FocusAdapter() {
      @Override
      public void focusLost(FocusEvent fe) {
        renameComplete();
      }
    });
    txtName.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent ae) {
        renameComplete();
      }
    });
  }
  
  public SubButton getRemoveButton() {
    return btnRemove;
  }
  
  public SubButton getRenameButton() {
    return btnRename;
  }
  
  public SubButton getAddButton() {
    return btnAdd;
  }
  
  public SubButton getPlayButton() {
    return btnPlay;
  }
  
  public void updateView() {
    lblName.setText(playlist.getName());
    lblNumSongs.setText(playlist.size() + " songs");
    if (worker != null) worker.cancel(false);
    // To avoid concurrency problems, we give the worker a copy of the array
    // to work with.
    worker = new CoverArtPainter(playlist.getFileArray());
    worker.execute();
  }
  
  @Override
  public void setHighlight(boolean highlight) {
    super.setHighlight(highlight);
    lblName.setFont(new Font("Microsoft YaHei",
        highlight ? Font.BOLD : Font.PLAIN, 15));
  }

  public Playlist getPlaylist() {
    return playlist;
  }
  
  public void performRename() {
    lblName.setVisible(false);
    txtName.setText(playlist.getName());
    txtName.setVisible(true);
    txtName.requestFocusInWindow();
  }
  
  private void renameComplete() {
    lblName.setVisible(true);
    txtName.setVisible(false);
    playlist.setName(txtName.getText());
  }
  
  class CoverArtPainter extends SwingWorker<Image, Void> {
    File[] files;
    private CoverArtPainter(File[] files) {
      this.files = files;
    }

    /**
     * This lengthy, lengthy method attempts to whip up an aggregate icon
     * with the 3 most frequently recurring cover arts in the playlist. We
     * don't have to worry about the lag that this lengthy, lengty method
     * causes, since it's all discreetly done in the background thanks to
     * our SwingWorker!
     * @return The whipped-up aggregate icon.
     */
    @Override
    protected Image doInBackground() {
      // If there are no songs or 1 song in this playlist, things get much easier.
      if (files.length == 0) {
        return ResourcePreloader.getDefaultCoverArt().getScaledInstance(120, 
            120, Image.SCALE_AREA_AVERAGING);
      } else if (files.length == 1) {
        return new SongMetadata(files[0]).getCoverArt().getScaledInstance(120,
            120, Image.SCALE_AREA_AVERAGING);
      }
      
      // Count the frequency of each existing album in this playlist.
      HashMap<String, AlbumCounterInfo> albumFrequency
          = new HashMap<String, AlbumCounterInfo>();
      for (File file : files) {
        if (isCancelled()) return null;
        SongMetadata metadata = new SongMetadata(file);
        
        if (!albumFrequency.containsKey(metadata.getAlbum())) {
          albumFrequency.put(metadata.getAlbum(), new AlbumCounterInfo(metadata.getAlbum()));
        }
        
        AlbumCounterInfo ac = albumFrequency.get(metadata.getAlbum());
        ac.frequency++;
        if (ac.hasNoArt()) {
          ac.coverArt = metadata.getCoverArt();
        }
      }
      
      // Get the 3 winner albums.
      Collection<AlbumCounterInfo> contestants = albumFrequency.values();
      Image[] winners = new Image[3];
      for (int i = 0; i < 3; i++) {
        if (isCancelled()) return null;
        AlbumCounterInfo winner = Collections.max(contestants);
        contestants.remove(winner);
        winners[i] = winner.coverArt;
        if (contestants.isEmpty()) break;
      }
      
      // Again, if there is only one album, things get much easier.
      if (winners[1] == null) return winners[0].getScaledInstance(120, 120,
          Image.SCALE_AREA_AVERAGING);
      
      // Here comes the hard part: when there is more than one album.
      BufferedImage bi = new BufferedImage(120, 120, BufferedImage.TYPE_INT_ARGB);
      Graphics2D g2d = bi.createGraphics();
      g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
          RenderingHints.VALUE_INTERPOLATION_BICUBIC);
      g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
          RenderingHints.VALUE_ANTIALIAS_ON);
      AffineTransform at = new AffineTransform();
      
      if (winners[2] == null) {
        // There are two albums in this playlist.
        
        // Paint the runner-up.
        BufferedImage tmp1 = new BufferedImage(85, 85, BufferedImage.TYPE_INT_ARGB);
        Graphics g1 = tmp1.getGraphics();
        g1.drawImage(winners[1].getScaledInstance(85, 85, 
            Image.SCALE_AREA_AVERAGING), 0, 0, null);
        g1.dispose();
        
        at.translate(0, 20);
        at.rotate(-.1);
        g2d.setPaint(new TexturePaint(tmp1, new Rectangle2D.Float(0, 0, 85, 85)));
        g2d.transform(at);
        g2d.fillRect(0, 0, 85, 85);
        try {
          g2d.transform(at.createInverse());
        } catch (NoninvertibleTransformException ex) {}
        
        // Paint the winner.
        BufferedImage tmp2 = new BufferedImage(93, 93, BufferedImage.TYPE_INT_ARGB);
        Graphics g2 = tmp2.getGraphics();
        g2.drawImage(winners[0].getScaledInstance(93, 93, 
            Image.SCALE_AREA_AVERAGING), 0, 0, null);
        g2.dispose();
        
        at.setToIdentity();
        at.translate(27, 5);
        at.rotate(.1);
        g2d.setPaint(new TexturePaint(tmp2, new Rectangle2D.Float(0, 0, 93, 93)));
        g2d.transform(at);
        g2d.fillRect(0, 0, 93, 93);
        
        g2d.dispose();
        return bi;
      } else {
        // There are at least 3 albums in this playlist.
        
        // Paint the thrid place winner.
        BufferedImage tmp1 = new BufferedImage(80, 80, BufferedImage.TYPE_INT_ARGB);
        Graphics g1 = tmp1.getGraphics();
        g1.drawImage(winners[2].getScaledInstance(80, 80, 
            Image.SCALE_AREA_AVERAGING), 0, 0, null);
        g1.dispose();
        
        at.translate(0, 30);
        at.rotate(-.1);
        g2d.setPaint(new TexturePaint(tmp1, new Rectangle2D.Float(0, 0, 80, 80)));
        g2d.transform(at);
        g2d.fillRect(0, 0, 80, 80);
        try {
          g2d.transform(at.createInverse());
        } catch (NoninvertibleTransformException ex) {}
        
        // Paint the runner-up.
        at.setToIdentity();
        BufferedImage tmp2 = new BufferedImage(85, 85, BufferedImage.TYPE_INT_ARGB);
        Graphics g2 = tmp2.getGraphics();
        g2.drawImage(winners[1].getScaledInstance(85, 85, 
            Image.SCALE_AREA_AVERAGING), 0, 0, null);
        g2.dispose();
        
        at.translate(10, 12);
        at.rotate(-.03);
        g2d.setPaint(new TexturePaint(tmp2, new Rectangle2D.Float(0, 0, 85, 85)));
        g2d.transform(at);
        g2d.fillRect(0, 0, 85, 85);
        try {
          g2d.transform(at.createInverse());
        } catch (NoninvertibleTransformException ex) {}
        
        // Paint the winner.
        BufferedImage tmp3 = new BufferedImage(90, 90, BufferedImage.TYPE_INT_ARGB);
        Graphics g3 = tmp3.getGraphics();
        g3.drawImage(winners[0].getScaledInstance(90, 90, 
            Image.SCALE_AREA_AVERAGING), 0, 0, null);
        g3.dispose();
        
        at.setToIdentity();
        at.translate(30, 5);
        at.rotate(.1);
        g2d.setPaint(new TexturePaint(tmp3, new Rectangle2D.Float(0, 0, 90, 90)));
        g2d.transform(at);
        g2d.fillRect(0, 0, 90, 90);
        
        g2d.dispose();
        return bi;
      }
    }
    
    @Override
    protected void done() {
      try {
        lblCoverArts.setIcon(new ImageIcon(get()));
      } catch (java.util.concurrent.CancellationException ex) {
      } catch (InterruptedException ex) {
      } catch (java.util.concurrent.ExecutionException ex) {
        System.err.println("WTF CONCURRENT EXECUTION EXCEPTION?!");
      }
    }
    
    private class AlbumCounterInfo implements Comparable<AlbumCounterInfo> {
      int frequency = 0;
      String album;
      Image coverArt;
      AlbumCounterInfo(String album) { this.album = album; }
      boolean hasNoArt() {
        return coverArt == null || coverArt.equals(ResourcePreloader.getDefaultCoverArt());
      }

      @Override
      public int compareTo(AlbumCounterInfo t) {
        if (frequency > t.frequency) return 1;
        if (frequency < t.frequency) return -1;
        return 0;
      }
    }
    
  }
}
