/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.server;

import java.net.*;
import java.io.*;

/**
 *
 * @author Wyverald
 */
public class ServerTalker extends Thread {
  private Socket socket;
  private File folder;
  private ServerFrame frame;
  
  public ServerTalker(Socket socket, File folder, ServerFrame frame) {
    this.socket = socket;
    this.folder = folder;
    this.frame = frame;
  }
  
  @Override
  public void run() {
    log("Checked in.");
    
    // Open input and output streams.
    DataInputStream is = null;
    try {
      is = new DataInputStream(socket.getInputStream());
    } catch (IOException ex) {
      log("Could not open input stream");
    }
    DataOutputStream os = null;
    try {
      os = new DataOutputStream(socket.getOutputStream());
    } catch (IOException ex) {
      log("Could not open output stream");
    }
    
    // Generate response.
    try {
      String command = is.readUTF();

      if (command.equals("list")) {
        // list(String query, int offset, int count) 
        //   => int total, int from, int to, (String item)+

        String query = is.readUTF();
        int offset = is.readInt();
        int count = is.readInt();
        log("Listing with query " + query + " offset " + offset + " count " + count);

        String[] files = folder.list(new ContainsFilter(query));
        os.writeInt(files.length);
        int from = offset, to = Math.min(files.length, offset + count - 1);
        os.writeInt(from);
        os.writeInt(to);
        for (int i = from - 1; i < to; i++) {
          os.writeUTF(files[i].substring(0, files[i].length() - 4));
        }
        os.flush();
        
        log(String.format("%d - %d of %d", from, to, files.length));
        
      } else if (command.equals("download")) {
        // download(String title, String artist, String album)
        //   => long size, byte[] buf
        
        String filename = is.readUTF() + "#" + is.readUTF() + "#" +
            is.readUTF() + ".mp3";
        log("Downloading file " + filename);
        
        File file = new File(folder, filename);
        FileRedirector.fileToDataOutputStream(file, os);
        FileRedirector.fileToDataOutputStream(findLyricsFileFromSongFile(file), os);
        os.flush();
      } else if (command.equals("upload")) {
        // upload(String title, String artist, String album, long size, byte[] buf)
        //   => boolean success
        
        String filename = is.readUTF() + "#" + is.readUTF() + "#" +
            is.readUTF() + ".mp3";
        log("Uploading file " + filename);
        
        File file = new File(folder, filename);
        if (file.exists()) {
          os.writeBoolean(false);
        } else {
          FileRedirector.dataInputStreamToFile(file, is);
          FileRedirector.dataInputStreamToFile(findLyricsFileFromSongFile(file), is);
          
          os.writeBoolean(true);
        }
        os.flush();
      }
    } catch (IOException ex) {
      log("Error conversing: " + ex);
    }
    
    // Cleanup.
    try {
      if (os != null) os.close();
      if (is != null) is.close();
      socket.close();
    } catch (IOException ex) {
      log("Error closing socket");
    }
    
    log("Checked out.");
  }
  
  private void log(String message) {
    frame.log("Talker " + getName() + ": " + message);
  }
  
  class ContainsFilter implements FilenameFilter {
    private String query;
    public ContainsFilter(String q) { query = q; }

    @Override
    public boolean accept(File file, String string) {
      return string.toLowerCase().contains(query.toLowerCase()) &&
          string.endsWith(".mp3");
    }
    
  }
  
  public static File findLyricsFileFromSongFile(File musicFile) {
    int lastDot = musicFile.getName().lastIndexOf(".");
    String lrcFilename = null;
    if (lastDot == -1) {
      lrcFilename = musicFile.getName() + ".lrc";
    } else {
      lrcFilename = musicFile.getName().substring(0, lastDot) + ".lrc";
    }
    return new File(musicFile.getParent(), lrcFilename);
  }
}
