/*
 * The MIT License
 *
 * Copyright 2011 Wyverald.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package melodia.util;

import java.io.*;
import org.mozilla.universalchardet.*;

/**
 * Reads a plain text file with a guessed encoding, utilizing Mozilla's
 * encoding guesser library.
 * @author Wyverald
 */
public class TextFileReader {
  private BufferedReader reader;
  public TextFileReader(File file) throws IOException {
    FileInputStream fis = new FileInputStream(file);
    byte[] buf = new byte[4096];
    
    // Detect character encoding with UniversalDetector.
    UniversalDetector detector = new UniversalDetector(null);
    int nread;
    while ((nread = fis.read(buf)) > 0 && !detector.isDone()) {
      detector.handleData(buf, 0, nread);
    }
    detector.dataEnd();
    
    String charset = detector.getDetectedCharset();
    fis.close();
    if (charset == null || charset.equalsIgnoreCase("Windows-1252")) {
      // If the charset could not be detected, we use GBK as a fallback.
      // If the charset detected is the standard ANSI one, we can also use
      // GBK as they are effectively the same.
      charset = "GBK";
    }

    reader = new BufferedReader(new InputStreamReader(
        new FileInputStream(file), charset));

    // UTF-8 BOM workaround
    if (charset.equalsIgnoreCase("UTF-8")) {
      reader.mark(5);
      if (reader.read() != 0xFEFF) reader.reset();
    }
  }
  
  public String readLine() throws IOException {
    return reader.readLine();
  }
  
  public String readAll() throws IOException {
    StringBuilder sb = new StringBuilder();
    char[] buffer = new char[4096];
    int len = 0;
    while ((len = reader.read(buffer, 0, 4096)) != -1) {
      sb.append(buffer, 0, len);
    }
    return sb.toString();
  }
  
  public void close() throws IOException {
    reader.close();
  }
}
